Do not base endgame bonus on arbitrary messages
[open-adventure.git] / score.c
diff --git a/score.c b/score.c
index 7b0269d8a1c87806cdcb5763945b9d04ff5c8120..8bd3cce1793af69a20c7b186a99ba58f7c27de1b 100644 (file)
--- a/score.c
+++ b/score.c
@@ -1,17 +1,18 @@
 #include <stdlib.h>
 #include "advent.h"
-#include "database.h"
-#include "newdb.h"
+#include "dungeon.h"
 
 /*
  * scoring and wrap-up
  */
 
-void score(enum termination mode)
+static long mxscor;    /* ugh..the price for having score() not exit. */
+
+long score(enum termination mode)
 /* mode is 'scoregame' if scoring, 'quitgame' if quitting, 'endgame' if died
  * or won */
 {
-    long score = 0, mxscor = 0;
+    long score = 0;
 
     /*  The present scoring algorithm is as follows:
      *     Objective:          Points:        Present total possible:
@@ -34,49 +35,56 @@ void score(enum termination mode)
 
     /*  First tally up the treasures.  Must be in building and not broken.
      *  Give the poor guy 2 points just for finding each treasure. */
-    for (long i=MINTRS; i<=MAXTRS; i++) {
-       if(object_descriptions[i].inventory != 0) {
-           long k=12;
-           if(i == CHEST)k=14;
-           if(i > CHEST)k=16;
-           if(game.prop[i] >= 0)
-               score += 2;
-           if(game.place[i] == LOC_BUILDING && game.prop[i] == 0)
-               score += k-2;
-           mxscor += k;
-       }
+    mxscor = 0;
+    for (int i = 1; i <= NOBJECTS; i++) {
+        if (!objects[i].is_treasure)
+            continue;
+        if (objects[i].inventory != 0) {
+            long k = 12;
+            if (i == CHEST)
+                k = 14;
+            if (i > CHEST)
+                k = 16;
+            if (game.prop[i] > STATE_NOTFOUND)
+                score += 2;
+            if (game.place[i] == LOC_BUILDING && game.prop[i] == STATE_FOUND)
+                score += k - 2;
+            mxscor += k;
+        }
     }
 
/*  Now look at how he finished and how far he got.  MAXDIE and
      *  game.numdie tell us how well he survived.  game.dflag will tell us
      *  if he ever got suitably deep into the cave.  game.closng still
      *  indicates whether he reached the endgame.  And if he got as far as
      *  "cave closed" (indicated by "game.closed"), then bonus is zero for
      *  mundane exits or 133, 134, 135 if he blew it (so to speak). */
-    score += (MAXDIE-game.numdie)*10;
-    mxscor += MAXDIE*10;
-    if(mode == endgame)
-       score += 4;
+    score += (NDEATHS - game.numdie) * 10;
+    mxscor += NDEATHS * 10;
+    if (mode == endgame)
+        score += 4;
     mxscor += 4;
-    if(game.dflag != 0)score += 25;
+    if (game.dflag != 0)
+        score += 25;
     mxscor += 25;
-    if(game.closng)score += 25;
+    if (game.closng)
+        score += 25;
     mxscor += 25;
-    if(game.closed) {
-       if(game.bonus == 0)
-           score += 10;
-       if(game.bonus == SPLATTER_MESSAGE)
-           score += 25;
-       if(game.bonus == DEFEAT_MESSAGE)
-           score += 30;
-       if(game.bonus == VICTORY_MESSAGE)
-           score += 45;
+    if (game.closed) {
+        if (game.bonus == none)
+            score += 10;
+        if (game.bonus == splatter)
+            score += 25;
+        if (game.bonus == defeat)
+            score += 30;
+        if (game.bonus == victory)
+            score += 45;
     }
     mxscor += 45;
 
     /* Did he come to Witt's End as he should? */
-    if(game.place[MAGZIN] == LOC_WITTSEND)
-       score += 1;
+    if (game.place[MAGAZINE] == LOC_WITTSEND)
+        score += 1;
     mxscor += 1;
 
     /* Round it off. */
@@ -84,43 +92,45 @@ void score(enum termination mode)
     mxscor += 2;
 
     /* Deduct for hints/turns/saves. Hints < 4 are special; see database desc. */
-    for (long i=1; i<=HNTMAX; i++) {
-       if(game.hinted[i])
-           score=score-HINTS[i][2];
+    for (long i = 0; i < NHINTS; i++) {
+        if (game.hinted[i])
+            score = score - hints[i].penalty;
     }
-    if(game.novice)
-       score -= 5;
-    if(game.clshnt)
-       score -= 10;
-    score=score-game.trnluz-game.saved;
+    if (game.novice)
+        score -= 5;
+    if (game.clshnt)
+        score -= 10;
+    score = score - game.trnluz - game.saved;
 
     /* Return to score command if that's where we came from. */
-    if(mode == scoregame) {
-       SETPRM(1,score,mxscor);
-       SETPRM(3,game.turns,game.turns);
-       RSPEAK(GARNERED_POINTS);
-       return;
+    if (mode == scoregame) {
+        rspeak(GARNERED_POINTS, score, mxscor, game.turns, game.turns);
     }
 
-    /* that should be good enough.  Let's tell him all about it. */
-    if(score+game.trnluz+1 >= mxscor && game.trnluz != 0)
-       RSPEAK(TOOK_LONG);
-    if(score+game.saved+1 >= mxscor && game.saved != 0)
-       RSPEAK(WITHOUT_SUSPENDS);
-    SETPRM(1,score,mxscor);
-    SETPRM(3,game.turns,game.turns);
-    RSPEAK(TOTAL_SCORE);
-    for (long i=1; i<=(long)CLSSES; i++) {
-       if(CVAL[i] >= score) {
-           newspeak(class_messages[i]);
-           i=CVAL[i]+1-score;
-           SETPRM(1,i,i);
-           RSPEAK(NEXT_HIGHER);
-           exit(0);
-       }
-    }
-    RSPEAK(OFF_SCALE);
-    RSPEAK(NO_HIGHER);
-    exit(0);
+    return score;
+}
 
+void terminate(enum termination mode)
+/* End of game.  Let's tell him all about it. */
+{
+    long points = score(mode);
+
+    if (points + game.trnluz + 1 >= mxscor && game.trnluz != 0)
+        rspeak(TOOK_LONG);
+    if (points + game.saved + 1 >= mxscor && game.saved != 0)
+        rspeak(WITHOUT_SUSPENDS);
+    rspeak(TOTAL_SCORE, points, mxscor, game.turns, game.turns);
+    for (long i = 1; i <= (long)NCLASSES; i++) {
+        if (classes[i].threshold >= points) {
+            speak(classes[i].message);
+            i = classes[i].threshold + 1 - points;
+            rspeak(NEXT_HIGHER, i, i);
+            exit(EXIT_SUCCESS);
+        }
+    }
+    rspeak(OFF_SCALE);
+    rspeak(NO_HIGHER);
+    exit(EXIT_SUCCESS);
 }
+
+/* end */
@@ -34,49 +35,56 @@ void score(enum termination mode)
     }
 
-    /*  Now look at how he finished and how far he got.  MAXDIE and
+    /*  Now look at how he finished and how far he got.  NDEATHS and
      *  game.numdie tell us how well he survived.  game.dflag will tell us
      *  if he ever got suitably deep into the cave.  game.closng still